-- HugeBagWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Habna

-- HugeBag Known bugs:
-- When right clicking on a tool the icon is not swaping properly
-- Dragging the mouse beyond the edge of LotRO in Window-mode while doing a resize will cause HugeBag to get out of the screen
-- Unload command not working
-- Reload command not working

-- HugeBag future functions:
-- Sorting Capability

-- HugeBag chage log:
-- 1.0.1 (2011/05/15) : added a context menu
-- 1.0.0 (2011/05/15) : Initial release

import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Extensions";
import "Turbine.UI.Lotro";
import "Turbine.Utils";

frmMain = class( Turbine.UI.Extensions.Window );

freeslots = 0; --> ** Global Variable to get the number of free slots **
backpackSize = self; --> ** Global variable to get the number of items in all 5 bags **
InitXPos = 17; --> ** Initial X position of the first Item in the window **
InitYPos = 33; --> ** Initial Y position of the first Item in the window **
width, height = self;
edgePadding = 15;
Version = "1.0.1";
StartVisible = true;
ShowSlotsInfo = true;
WinLocX, WinLocY = self;
WinWidth, WinHeight = self;
settings = self;

-- **v Constructor v**
function frmMain:Constructor()
	Turbine.UI.Extensions.Window.Constructor( self );
	
	self:LoadSetting();
	self:SetVisible( StartVisible );

	-- **v Setting to position the window at the bottom right of the screen v**
	self:SetSize( WinWidth, WinHeight );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetPosition( WinLocX, WinLocY );
	self:SetOpacity( 1.0 );
	self:SetAllowDrop( true );
	self:SetFadeSpeed( 0.4 );
	-- **^
	-- **v Set the opacity when the mouse enter the window v**
	self.MouseEnter = function( sender, args )
		sender:SetOpacity( 1.0 );
	end
	-- **^
	-- **v Set the opacity when the mouse leave the window v**
	self.MouseLeave = function( sender, args )
		sender:SetOpacity( 1.0 );
	end
	-- **^

	self:SetWantsKeyEvents( true );
	
	-- **v Detect key press v**
	self.KeyDown = function( sender, args )
		-- **v Hide windows if ESC key is press v**
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			
		end
		-- **^
		-- **v Show/Hide window if any bag was toggle v**
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) 
		end
		-- **^
	end
	-- **^
	-- **v Set the scrollbar v**
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );
	-- **^
	-- **v Set the resizehandle v**
	self.resizeHandle = Turbine.UI.Control();
	self.resizeHandle:SetParent( self );
	self.resizeHandle:SetZOrder( 100 );
	self.resizeHandle:SetSize( 20, 20 );
	self.resizeHandle:SetPosition( self:GetWidth() - self.resizeHandle:GetWidth(), self:GetHeight() - self.resizeHandle:GetHeight() );

	self.resizeHandle.MouseDown = function( sender, args )
		sender.dragStartX = args.X;
		sender.dragStartY = args.Y;
		sender.dragging = true;
	end

	self.resizeHandle.MouseUp = function( sender, args )
		sender.dragging = false;
		frmMain:SaveSetting()
	end

	self.resizeHandle.MouseMove = function( sender, args )
		width, height = self:GetSize();

		if ( sender.dragging ) then
			self:SetSize( width + ( args.X - sender.dragStartX ), height + ( args.Y - sender.dragStartY ) );
			sender:SetPosition( self:GetWidth() - sender:GetWidth(), self:GetHeight() - sender:GetHeight() )
			self:PerformLayout()
		end
	end
	-- **^
	-- **v Set the listbox v**
	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	-- **^	
			
	self.items = { }; --> ** Initialize the container that holds the elements referencing the player's inventory **

	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		player = Turbine.Gameplay.LocalPlayer.GetInstance();
	else
		player = Turbine.Gameplay.LocalPlayer();
	end
	-- **^

	self.backpack = player:GetBackpack(); --> ** Initialize the backpack with the current player gameplay **

	self:Refresh(); --> ** Refresh the backpack list **

	-- **v Backpack size has changed v**
	self.backpack.SizeChanged = function( sender, args )
		self:Refresh();
	end
	-- **^
	-- **v Adding an item v**
	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots -1;
		self:PrintTitle()
	end
	-- **^
	-- **v Removing an item v**
	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots +1;
		self:PrintTitle()
	end
	-- **^
	-- **v Changing place an item v**
	self.backpack.ItemMoved = function( sender, args )
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
	end

	self.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();

		if ( shortcut ~= nil ) then
			local destinationItemControl = self.itemListBox:GetItemAt( args.X - InitXPos, args.Y - InitYPos );

			-- if nil, then it's out of range, so don't do anything
			if ( destinationItemControl ~= nil ) then
				local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );
				self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
			end
		end
	end
	-- **^
	-- **v Context menu settings v**
	self.menu = Turbine.UI.ContextMenu()
	self.menu.items = self.menu:GetItems()
	--self.menu:SetOpacity( 1.0 );

	local option_line = Turbine.UI.MenuItem("-------------------------------------------------------------------", false, "line")

		local option_version = Turbine.UI.MenuItem("Show version of HugeBag in shell", true, "version")
	option_version.Click = VersionInfo

	local option_authors = Turbine.UI.MenuItem("Show authors of HugeBag in shell", true, "authors")
	option_authors.Click = AuthorsInfo

	local option_help = Turbine.UI.MenuItem("Show HugeBag commands", true, "help")
	option_help.Click = HelpInfo

	local option_reload = Turbine.UI.MenuItem("Reload HugeBag plugin instance", true, "reload")
	option_reload.Click = ReloadHugeBag

	local option_unload = Turbine.UI.MenuItem("Unload HugeBag plugin from memory", true, "unload")
	option_unload.Click = UnloadHugeBag

	local option_reset = Turbine.UI.MenuItem("Reset all settings to default", true, "reset")
	option_reset.Click = ResetSettings

	local option_slotsinfo = Turbine.UI.MenuItem("Show free, used and max slots in shell", true, "slotsinfo")
	option_slotsinfo.Click = SlotsInfo

	local option_toggleslots = Turbine.UI.MenuItem("Toggle slots informations in title", true, "toggleslots")
	option_toggleslots.Click = ToggleSlotsInfo

	local option_toggleshow = Turbine.UI.MenuItem("Toggle the startup visibility of HugeBag", true, "toggleshow")
	option_toggleshow.Click = ToggleShow

	self.menu.items:Add(option_slotsinfo)
	self.menu.items:Add(option_toggleshow)
	self.menu.items:Add(option_toggleslots)
	self.menu.items:Add(option_reset)
	self.menu.items:Add(option_line)
	self.menu.items:Add(option_version)
	self.menu.items:Add(option_authors)
	self.menu.items:Add(option_help)
	self.menu.items:Add(option_line)
	self.menu.items:Add(option_reload)
	self.menu.items:Add(option_unload)
	-- **^
end
-- **^
-- **v Show Help Info v**
function ReloadHugeBag()
	--Turbine.Shell.Execute( "/plugins load hugebag\r" );
	--Turbine.Shell.WriteLine( "HugeBag: loaded" );
	Turbine.Shell.WriteLine( "HugeBag: This function is currently not working" );
end
-- **^
-- **v Show Help Info v**
function UnloadHugeBag()
	--Turbine.Shell.Execute( "/plugins unload hugebag\r" );
	--Turbine.Shell.WriteLine( "HugeBag: unloaded" );
	Turbine.Shell.WriteLine( "HugeBag: This function is currently not working" );
end
-- **^
-- **v Show Version v**
function VersionInfo()
	Turbine.Shell.WriteLine( "HugeBag: The current version is: v" .. Version );
end
-- **^
-- **v Show Authors v**
function AuthorsInfo()
	Turbine.Shell.WriteLine( "HugeBag: The current authors are: Rod, Extended by Habna" );
end
-- **^
-- **v Show Help Info v**
function HelpInfo()
	Turbine.Shell.WriteLine( "\n ** HugeBag v" .. Version .. " commands **\ntoggle: Toggle visibility on and off, don\'t use 'show' or 'hide'\nreset: Reset location, size and visibility to default\nslotsinfo: Show free, used and max slots in shell\nreload: Reload HugeBag plugin\nunload/doom: Unload HugeBag plugin from memory\ntoggleslots: show or hide slots information in title\ntoggleshow: show or hide HugeBag at startup\nhelp/?: Show HugeBag commands" );	
end
-- **^
-- **v Show Slots Info v**
function SlotsInfo()
	local usedslots = backpackSize - freeslots;
	Turbine.Shell.WriteLine( "HugeBag: I have " .. freeslots .. " free slots, " .. usedslots .. " used slots and " .. backpackSize .. " max slots" );
end
-- **^
-- **v Toggle Startup Visibility v**
function ToggleShow()
	StartVisible = ( not StartVisible );
	frmMain:SaveSetting()
	if StartVisible == false then
		Turbine.Shell.WriteLine( "HugeBag: will load hidden next time" );
	else
		Turbine.Shell.WriteLine( "HugeBag: will load visibile next time" );
	end
end
-- **^
-- **v Toggle Slots Info v**
function ToggleSlotsInfo()
	ShowSlotsInfo = ( not ShowSlotsInfo );
	frmMain:SaveSetting()
	frmMain:PrintTitle()
	if ShowSlotsInfo == true then
		Turbine.Shell.WriteLine( "HugeBag: now showing slots informations in the title" );
	else
		Turbine.Shell.WriteLine( "HugeBag: not showing slots informations in the title" );
	end
end
-- **^
-- **v Reset Settings v**
function ResetSettings()
	StartVisible = true;
	ShowSlotsInfo = true;
	frmMain:SetSize( 460, 300 );
	frmMain:SetPosition( Turbine.UI.Display.GetWidth() - 15 - 460, Turbine.UI.Display.GetHeight() - 15 - 300 - 55 );
	frmMain:SaveSetting()
	frmMain:SetVisible( true );
	frmMain:PerformLayout()
	Turbine.Shell.WriteLine( "HugeBag: all settings was set back to default" );
end
-- **^
-- **v Stuff to do when mouse is down on the form border v**
function frmMain:MouseDown(args)
	if args.Button == Turbine.UI.MouseButton.Left then
			
	elseif args.Button == Turbine.UI.MouseButton.Right then
		self.menu:ShowMenu()
	end
end
-- **^
-- **v Refresh the X,Y positiov when window position change v**
function frmMain:PositionChanged(args)
	local left, top = self:GetPosition()

	WinLocX = left
	WinLocY = top

	frmMain:SaveSetting()
end
-- **^
-- **v Refresh the backpack list v**
function frmMain:Refresh()
	backpackSize = self.backpack:GetSize();
	local temp;

	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end

		-- Get item and set background color
		temp = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		temp:SetBackground( "HabnaPlugins/HugeBag/Resources/slotBackground.tga" );
		temp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		
		self.items[i] = temp;
		--self.items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		self.itemListBox:AddItem( self.items[i] );
	end
	self:PerformLayout();
end
-- **^
-- **v Layout of the items in the listbox v**
function frmMain:PerformLayout()
	width, height = self:GetSize();
	WinLocX, WinLocY = self:GetPosition()
	local itemWidth = 40;
	
	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = width - 40;
	local listHeight = height - 50;
	local itemsPerRow = listWidth / itemWidth;
	
	self.itemListBox:SetPosition( InitXPos, InitYPos );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	self.itemListBoxScrollBar:SetPosition( width - 17, 40 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );

	self:CountFreeSlots()
end
-- **^
-- **v Do stuff before closing the windows v**
function frmMain:Closing(args)
	self:SaveSetting();
end
-- **^
-- **v Count the free slots v**
function frmMain:CountFreeSlots()
	freeslots = 0;

	for i = 1, backpackSize, 1 do
		if ( self.backpack:GetItem( i ) == nil ) then
			freeslots = freeslots + 1;
		end
	end

	self:PrintTitle()
end
-- **^
-- **v Print the title with the free slots and the max v**
function frmMain:PrintTitle()
	if ShowSlotsInfo == true then
		self:SetText( "HugeBag - free slots: " .. freeslots .. " / " .. backpackSize );
	else
		self:SetText( "HugeBag" );
	end
end
-- **^
-- **v Load settings v**
function frmMain:LoadSetting()
	settings = {};
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "HugeBagSettings" );
	
	if settings == nil then
		WinWidth = 460;
		WinHeight = 300;
		self:SetSize( WinWidth, WinHeight );
		WinLocX = Turbine.UI.Display.GetWidth() - edgePadding - WinWidth
		WinLocY = Turbine.UI.Display.GetHeight() - edgePadding - WinHeight - 55
		self:SaveSetting()
	end

	Version = settings.Version;
	StartVisible = settings.StartVisible;
	ShowSlotsInfo = settings.ShowSlotsInfo;
	WinLocX = settings.X;
	WinLocY = settings.Y;
	WinWidth = settings.Width;
	WinHeight = settings.Height;
end
-- **^
-- **v Save settings v**
function frmMain:SaveSetting()
	settings = {};
	settings.Version = Version;
	settings.StartVisible = StartVisible;
	settings.ShowSlotsInfo = ShowSlotsInfo;
	settings.X = WinLocX; --Turbine.UI.Display.GetWidth() - edgePadding - self:GetWidth()
	settings.Y = WinLocY; --Turbine.UI.Display.GetHeight() - edgePadding - self:GetHeight() - 55
	settings.Width = self:GetWidth();
	settings.Height = self:GetHeight();

	Turbine.PluginData.Save( Turbine.DataScope.Character, "HugeBagSettings", settings );
end
-- **^